(in-package ocml)

;; +++++++++++++
;; the PATHWAYS : generic functions
;; +++++++++++++



(defvar *pathways-list* 
;;"Here we manually add all the info about the pathways created - the last item is the class of entities good for the pathway"

  '((th-ideas-with-same-names "Ideas having the same name" 
                              "<b>Type of pathway</b>: THEORETICAL <br><b>Description</b>    : This pathway populates the graph with ideas having the same name but a different meaning than the selected one"
                              propositional-content)

    (th-views-map "Generic and specific schools of thought" 
                              "<b>Type of pathway</b>: THEORETICAL <br><b>Description</b>    : Starting from a school of thought, this pathway retrieves a set of related schools of thought which are all specializations of the same general one"
                               SCHOOL-OF-THOUGHT)

    (th-views-influences "Influences among related views" 
                              "<b>Type of pathway</b>: THEORETICAL <br><b>Description</b>    : Starting from a view, this pathway is a recursive functon showing information about others views that compete/support with the first one."
                              view)

    (th-generic-ideas-map  "Generic map of related ideas" 
                       "<b>Type of pathway</b>: THEORETICAL <br><b>Description</b>    : This pathway shows all the general information stored about an idea"
                       propositional-content)

(th-problem-map "Problem-centric map of the attempt to solve a problem" 
                "<b>Type of pathway</b>: THEORETICAL <br><b>Description</b>    : This pathway takes a problem instance and retrieves information related to the competing views (theories, schools of thought, philosophies) that tackle and attempt to solve the problem"
                       problem)

    
    (txt-also-has-idea-as-subject "Other documents about the same idea"  
                                  "<b>Type of pathway</b>: THEORETICAL <br><b>Description</b>    : This pathway retrieves other information objects (beyond the tractatus) which deal with a specific idea, only by using stored semantic relationships" 
                                  propositional-content)

    (txt-strict-string-matching "Strict string-matching on other resources"  
                                "<b>Type of pathway</b>: TEXTUAL <br><b>Description</b>    : This pathway combines semantic and syntactic data; it displays other resources where the value of the <i>is about</i> attribute matches <i>almost exactly</i> the name of the content searched. For this reason, results are not always relevant.." 
                                CRM-ENTITY)

    (txt-non-strict-string-matching "Non-strict string-matching on other resources"  
                                    "<b>Type of pathway</b>: TEXTUAL <br><b>Description</b>    : This pathway combines semantic and syntactic data; it displays other resources where the value of the <i>is about</i> attribute matches <i>at least 50 per cent</i> of the name of the content searched. This pathway may produce a large number of unwanted results." 
                                    CRM-ENTITY)
    
    (hs-advisors-tree  "Phd advisors" 
                       "<b>Type of pathway</b>: HISTORICAL <br><b>Description</b>    : This pathway shows the chain of phd-advisors starting from a selected philosopher or scientist"
                       PERSON)

    (hs-phd-students  "PhD students"   
                      "<b>Type of pathway</b>: HISTORICAL  <br><b>Description</b>    : This pathway shows the chain of phd-students starting from a selected philosopher or scientist" 
                      PERSON)
    
    (geo-persons-birthplace-same   "Scholars born in same area"  
                                   "<b>Type of pathway</b>: GEOGRAPHICAL <br><b>Description</b>    : This pathway retrieves information about the birthpace of a person, and shows which other scholars come from the same area."
                                   PERSON)
))



(defun check-available-pathways (instance)
"Check well my friend"
  (let ((result '()))
    (dolist (pathway *pathways-list*)
      (if (funcall (first pathway) instance)
          (push (first pathway) result)))
    result))
       

(defun this-pathway-has-results (pathway-shortname instance)
"Just checking a single pathway on an instance"
(let ((result '()))
  (if (funcall pathway-shortname instance)
      (push (funcall pathway-shortname instance) result))
  result))
 


(defun explode-pathway (implicit-pathway type)
"Modifies since now we have two visualization: this one produces a simple structure....."
  (let ((out '()))
  (cond

   ((eq type 'th-ideas-with-same-names)
    (dolist (item implicit-pathway)
      (push (list 'has-common-name (list (first item) (common-name (first item)))) out)
      (push (list 'defined-by-view (list (first item) (second item))) out)))

   ((eq type 'th-generic-ideas-map)
    (let ((original-idea (first implicit-pathway)))
      (dolist (item (rest implicit-pathway))
        (dolist (item2 (second item))
          (push (list (first item)  (list original-idea item2)) out)))))

   ((eq type 'th-views-map)
    (setf out implicit-pathway))
   
   ((eq type 'th-problem-map)
    (setf out implicit-pathway))

   ((eq type 'th-views-influences)
    (setf out implicit-pathway))
          
   ((eq type 'txt-strict-string-matching)
    (let ((original-string (first implicit-pathway)))
      (dolist (item (rest implicit-pathway))
        ;;(push (list 'has-uri    (list (common-name (second item)) (first (third item)))) out)  --> confuses the system
        (push (list 'has-publisher (list (second item) (first (third item)))) out)
        (push (list 'similar-to (list original-string (second item))) out))))

   ((eq type 'txt-non-strict-string-matching)  ;;same as above, for now
     (let ((original-string (first implicit-pathway)))
      (dolist (item (rest implicit-pathway))
        ;;(push (list 'has-uri    (list (common-name (second item)) (first (third item)))) out)
        (push (list 'has-publisher (list (second item) (first (third item)))) out)
        (push (list 'similar-to (list original-string (second item))) out))))

   ((eq type 'hs-advisors-tree)
    (dotimes (n (length implicit-pathway))  ;;they are persons
      (if (not (equal (nth n implicit-pathway) (first (last implicit-pathway))))
          (push (list 'had-PhD-advisor (list (nth n implicit-pathway) (nth (+ 1 n) implicit-pathway))) out))))

   ((eq type 'hs-phd-students)
    (dotimes (n (length implicit-pathway))  ;;they are persons
      (if (not (equal (nth n implicit-pathway) (first (last implicit-pathway))))
          (push (list 'had-PhD-student (list (nth n implicit-pathway) (nth (+ 1 n) implicit-pathway))) out)))))
  out))



(defun graph-explode-pathway (implicit-pathway type)
"For every pathway, explodes all the implicit information represented in the output of one of the main pathway-generation functions. The 'explosion' format must be a list of lists with the structure <relation> <term1 term2>. This is needed so to guarantee the functioning of the <build-graph-components> function which provides the right input to the applet. Another IMPORTANT thing to remember is that we should avoid to output common-names or strings, cause then they cannot be re-used as instances by selecting them on the graph!!!!!"
  (let ((out '()))
  (cond

   ((eq type 'th-ideas-with-same-names)
    (dolist (item implicit-pathway)
      (push (list 'has-common-name (list (first item) (common-name (first item)))) out)
      (push (list 'defined-by-view (list (first item) (second item))) out)))

   ((eq type 'th-generic-ideas-map)
    (let ((original-idea (first implicit-pathway)))
      (dolist (item (rest implicit-pathway))
        (dolist (item2 (second item))
          (push (list (first item)  (list original-idea item2)) out)))))

   ((eq type 'th-views-map)
    (let ((generic-school (first (first implicit-pathway))))
      (dolist (item (first (rest implicit-pathway)))
        (push (list 'has-description (list (first item) (string-description (first item)))) out)
        (push (list 'related-to-school (list (first item) generic-school)) out)
        (dolist (item2 (second item))
           (push (list 'exists-in-area (list (first item) item2)) out)))
      (push (list 'has-description (list generic-school (string-description generic-school))) out)))

   ((eq type 'th-problem-map)
    ())

   ((eq type 'th-views-influences)
    ())

          
   ((eq type 'txt-strict-string-matching)
    (let ((original-string (first implicit-pathway)))
      (dolist (item (rest implicit-pathway))
        ;;(push (list 'has-uri    (list (common-name (second item)) (first (third item)))) out)  --> confuses the system
        (push (list 'has-publisher (list (second item) (first (third item)))) out)
        (push (list 'similar-to (list original-string (second item))) out))))

   ((eq type 'txt-non-strict-string-matching)  ;;same as above, for now
     (let ((original-string (first implicit-pathway)))
      (dolist (item (rest implicit-pathway))
        ;;(push (list 'has-uri    (list (common-name (second item)) (first (third item)))) out)
        (push (list 'has-publisher (list (second item) (first (third item)))) out)
        (push (list 'similar-to (list original-string (second item))) out))))

   ((eq type 'hs-advisors-tree)
    (dotimes (n (length implicit-pathway))  ;;they are persons
      (if (not (equal (nth n implicit-pathway) (first (last implicit-pathway))))
          (push (list 'had-PhD-advisor (list (nth n implicit-pathway) (nth (+ 1 n) implicit-pathway))) out))))

   ((eq type 'hs-phd-students)
    (dotimes (n (length implicit-pathway))  ;;they are persons
      (if (not (equal (nth n implicit-pathway) (first (last implicit-pathway))))
          (push (list 'had-PhD-student (list (nth n implicit-pathway) (nth (+ 1 n) implicit-pathway))) out)))))
  out))
          






;;++++
;; pathway generation functions   
;; rationale: the output has just the minimum/critical info, leaving implicit for the exploding function all the rest
;;++++


;; {{{{{{{{{{{{ theoretical
(defun th-ideas-with-same-names (idea)
"Works mostly for concepts (but not only - it could be expanded with some regex function that looks a similar theories names..): from an idea-instance ----> outputs other ideas having the same name, with the respective defining views"
(if (has-granfather? idea 'concept)
    (let ((other-ideas (find-ideas-with-name (common-name idea)))
          (p '()))
      (if other-ideas
          (dolist (eachone other-ideas)
            (push (list eachone (first (my-slot-values eachone 'defined-by-view))) p)))
      p)))
    


(defun th-generic-ideas-map (idea)
  (if (has-granfather? idea 'propositional-content)
      (let ((related-ideas (find-idea-interpretation-content idea)))
        (if related-ideas
            (append (list idea)
                    (cl-user::reorder-list (cl-user::breakdown-list related-ideas)))))))


;; recursive results
(defun th-views-influences (view)
(let ((out '())
      (level 0)
      (maxlevel 2))
  (setf out (th-views-influences-internal view level maxlevel))
  out))

   
;; thing to add: not include duplicates!!    
(defun th-views-influences-internal (view level maxlevel)
(let ((out '())
      (ints (find-idea-interpretation-content view)))
  (if ints
      (dolist (int (first ints))
        (let ((rel (first int))
              (vals (second int)))
          (if (or (equal rel 'INFLUENCES-VIEW)
                  (equal rel 'INFLUENCED-BY-VIEW)
                  (equal rel 'OPPOSES-VIEW)
                  (equal rel 'SUPPORTS-VIEW))
              (push (list rel 
                          (let ((rec '()))
                            (dolist (val vals)
                              (push (list val 
                                          (if (< level maxlevel)
                                              (th-views-influences-internal val (+ 1 level) maxlevel)
                                            nil)) rec))
                            rec))
                            
                           out)))))
  out))

      


;; gets the views that tackle a problem, and then at a nested level the interpretations on such views...
(defun th-problem-map (problem)
  (if (has-granfather? problem 'problem)
        (let ((ints (find-idea-interpretation-content problem))
              (out '()))
          (if ints
              (let ((goodints (cl-user::breakdown-list ints)))
                (dolist (gd goodints)
                  ;;(setf out (append out (list (first gd))))
                  (let ((temp (list (first gd))))
                    (dolist (valori (second gd))
                      (let ((ints2 (find-idea-interpretation-content valori)))
                        (if ints2
                            (push (list valori (cl-user::breakdown-list ints2)) temp)
                          (push (list valori ints2) temp))))
                    (setf out (append  out (list temp)))))))
                  
          (if out
              (list problem out)
            out))))



(defun th-views-map (idea)
  (let (out '())
    (cond 
     ;; context. schools
     ((has-father? idea 'CONTEXTUALIZED-SCHOOL-OF-THOUGHT)
           (let ((generic-school (setofall '?x `(and (SCHOOL-OF-THOUGHT ?x) (is-about-school ,idea ?x)))))
             (if generic-school
                 (let* ((new-idea (first generic-school))
                        (related-schools (setofall '?x `(and (SCHOOL-OF-THOUGHT ?x) (is-about-school ?x ,new-idea)))))
                   (if related-schools
                        (let ((out2 '()))
                              (dolist (school related-schools)
                                (let ((areas (setofall '?x `(exists-in-area ,school ?x)))
                                      (ints (find-idea-interpretation-content school)))
                                  (if ints
                                      (setf out2 (append (list (list school areas (cl-user::breakdown-list ints))) out2))
                                    (setf out2 (append (list (list school areas)) out2)))))                 
                              (let ((new-idea-ints (find-idea-interpretation-content new-idea)))
                                (if new-idea-ints
                                    (setf out (list (list new-idea 
                                                          (cl-user::breakdown-list new-idea-ints )) out2))
                                  (setf out (list (list new-idea) out2))))))))))
     ;; schools            
     ((has-father? idea 'SCHOOL-OF-THOUGHT)
           (let ((related-schools (setofall '?x `(and (SCHOOL-OF-THOUGHT ?x) (is-about-school ?x ,idea)))))
             (if related-schools
                 (let ((out2 '()))
                   (dolist (school related-schools)
                     (let ((areas (setofall '?x `(exists-in-area ,school ?x)))
                           (ints (find-idea-interpretation-content school)))
                       (if ints
                           (setf out2 (append (list (list school areas (cl-user::breakdown-list ints))) out2))
                         (setf out2 (append (list (list school areas)) out2)))))
                   (let ((idea-ints (find-idea-interpretation-content idea)))
                     (if idea-ints
                         (setf out (list (list idea (cl-user::breakdown-list idea-ints )) out2))
                       (setf out (list (list idea) out2))))))))

                   ;(setf out (list idea out2))))))
     
     ;;to do?      
     ((has-father? idea 'THEORY) 
           ()))
    out))
      
  





;; {{{{{{{{{{{{{  textual
(defun txt-also-has-idea-as-subject (idea)
"Retrieves other information objects (beyond the tractatus) which deal with a specific idea"
(if (has-granfather? idea 'propositional-content)
    (let ((other-docs (setofall '?x `(and (io-interpretation ?i) (has-interpretation ?i ,idea) (interprets ?i ?x) (not (PART-OF-EXPRESSION ?x Tractatus-pears-english-version )))))
          (out '()))
      (if other-docs
          (setf out (list idea other-docs)))
      out)))
  
  




;; the following two have this type of output: 
;; e.g. (FACT ("Act" DICTIONARY-OF-PHILOSOPHY-PUBLICATION-G2741 (DICTIONARY-OF-PHILOSOPHY-WEBSITE)) ("Act  Psychology" DI....
(defun txt-strict-string-matching (idea)
  (let* ((idea-name (if (common-name idea)
                       (common-name idea)
                     (string idea))) ;; double check
        (results (word-also-dealt-with-in idea-name 50)))
    (if results
        (push idea-name results))))


(defun txt-non-strict-string-matching (idea)
  (let* ((idea-name (if (common-name idea)
                       (common-name idea)
                     (string idea)))
        (results (word-also-dealt-with-in idea-name 0)))
      (if results
          (push idea-name results))))














;; {{{{{{{{{{{{  historical

;; you need to load the people KB first...
(defun hs-advisors-tree (person)
"Will have to extend it: now it outputs a list, first the starting person, then the ascending tree"
(if (has-granfather? person 'person)
    (let ((tree (next-advisor-up person))
          (out nil))
      (if tree
          (setf out (append (list person) tree)))
      out)))


(defun hs-phd-students (person)
"To be extended"
(if (has-granfather? person 'person)
    (let ((students (all-phd-students person))
          (out nil))
      (if students
          (setf out (append (list person) students)))
      out)))







;; {{{{{{{{{{{{  geographical


(defun geo-persons-birthplace-same (person)
  (if (has-granfather? person 'person)
      (let ((birthplace (first (my-slot-values person 'has-birth-place)))
            (out nil))
        (if birthplace
            (let ((neighbours (setofall '?x `(and (person ?x) (has-birth-place birthplace)))))
              (if neighbours
                  (setf out neighbours))))
        out)))
