;;; -*- Mode: LISP; Syntax: Common-lisp; Base: 10; Package: OCML;   -*-

;; ++Philosurfical-CRM ontology

;; here i put all the philosophical ideas ---> all the abstract types in the philosophical domain!


 
(in-package "OCML")


 

(def-class Philosophical-idea (product-proposition)
"++PhiloSURFical : a philosophical idea is a propositional content with a specific importance within the philosophical world. Philosophical ideas are usually product propositions, as they are important in themselves, for argumentation or theoretical purposes, and not for specifying an action. The class methods is kind of borderline - future refinements should address this!"
((has-description :type String) 
 (has-common-name :type idea-appellation)
 ;;(appears-in :type Information-object)
 (has-referred-author :type Actor))
:slot-renaming ((has-referred-author was-made-by))) ;;for clarity reasons




;;;;;;;;;;;;;;;;;;;    V I E W   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



(def-class View (Philosophical-idea)
"++PhiloSURFical : Generic class referring to propositions expressing a viewpoint, that is, descriptions defining other concepts and, in general, meanings. The property exists-in-area can have multiple values.Defines-method is excluded at this level cause a thesis is not enough to define a method. Another possible relationis defines-key-concept..."
((exists-in-area :type Problem-area)
 (interprets-fact :type temporal-entity)
 (typifies :type intellectual-movement)))



(def-class Thesis (View)
"++PhiloSURFical : Philosophical-idea expressing a standpoint in need of demonstration but not necessarily having it"
((part-of-thesis :type Thesis)
 (part-of-theory :type Theory)
 (part-of-system :type Philosophical-system)))

(def-class Law (Thesis)
"++PhiloSURFical : A thesis with vast predictive power, especially in scientific areas")
(def-class Principle (Thesis)
"++PhiloSURFical : It is a thesis demonstrated or taken as fundamental in a philosophical system, e.g. the -ego- in the -idealism-")
(def-class Self-evident-principle (Principle)
"++PhiloSURFical : A principle that is self-demonstrated, so it's a valid assumption")



(def-class Theory (View)
"++PhiloSURFical : A systemic conceptual construction, with a coherent and organic architecture. It explains a specific phenomena (or set of) and answers to an existing problem. It comprises concepts, propositions and it HAS to be related to at least one argument. This is required to guarantee a more solid architecture, compared for example to a School-of-thought. Examples are the -theory of evolution- or the -theory of possible worlds-."
((part-of-theory :type Theory)
 (part-of-system :type Philosophical-system)
 (defines-Method :type Method)
 (has-thesis :type Thesis)))

(def-class Philosophical-theory (theory)
"++PhiloSURFical : a theory exixting within some philosophical area of research"
((exists-in-area :type branch-of-philosophy)))

(def-class Scientific-theory (theory)
"++PhiloSURFical : "
((exists-in-area :type Scientific-discipline)
 (predicts-fact :type temporal-entity)
 (verified-by-fact :type temporal-entity)))




(def-class Philosophical-system  (view)
"++PhiloSURFical : It is the Philosophical-idea comprising the set of an author's theories, within a system. A philosopher can build different ph. systems in his life, see Wittgenstein for example"
((defines-method :type method)
 (has-theory :type theory)
 (has-thesis :type Thesis)))



(def-class School-of-thought  (View)
"++PhiloSURFical : A generic standpoint, a belief or view about a subject area or a problem. It is not as formalized and systematic as a theory, and its contents are limited to be thesis. Examples are -pacifism-, -animism-, -expansionism-. They could be further classified depending on their area of provenience (politics, religion). This class is needed cause often a mental content refer to a belief, but without the specificity of a theory"
((has-main-thesis :type Thesis)
 (has-exemplar-theory :type Theory) ;; the theory that has inspired it, or helps understanding it
 (classifies-view :type View)))

(def-class Contextualized-school-of-thought (School-of-thought)
"++PhiloSURFical : Needed to separate the meaning of generic schools or conceptions, to their related but more specific meanings in specific fields of study"
((is-about-school :type School-of-thought)
 (exists-in-area :type Field-of-study)))




;;;;;;;;;;;;;;;;; C O N C E P T ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(def-class Concept (Philosophical-idea)
"++PhiloSURFical : atomic propositional content, in relation to a view defining it. It is recognizable because it composes the building material for a view, and acquires meaning from it"
((defined-by-view :type View)           
 (is-specialization-of :type Concept)  ;; narrower than
 (is-generalization-of :type Concept)  ;; broader than
 (is-equivalent-to :type Concept)  ;; has the same meaning
 (has-opposite-concept :type Concept)    ;; the classic antynomyc relation (--> dychotomy) .. wecould also specify more the various types of antynomy (gradable, relational, complementary)
 (has-related-concept :type Concept)  ;; reflects a generic and positive semantic closure
 (requires-concept :type concept)  ;; notional dependency rule, semantic dependance, implication such as "buy" and "pay"
 (causes-concept :type concept))) ;; e.g. to kill-to die not sure about this one



(def-class principial-entity (concept)
"++PhiloSURFical : ")
(def-class supernatural-entity (concept)
"++PhiloSURFical : ")
(def-class god ( principial-entity  supernatural-entity)
"++PhiloSURFical : ")

;; (def-class task-of-philosophy (concept))




;;;;;;;;;;;;; D I S T I N C T I O N ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(def-class Distinction (Philosophical-idea)
"++PhiloSURFical : "
((exists-in-area :type problem-area)
 (related-to-problem :type problem)
 (defined-by-view :type view)
 (contains-concept :type concept)))


(def-class Dichotomy (Distinction)
"++PhiloSURFical : A classification that states the breakdown of a space between two opposing principles or concepts, e.g.mind/body, relations-of-ideas/matters-of-fact in Hume, a-priori/a-posteriori in Kant "
((contains-concept :type concept :cardinality 2)))  







;;;;;;;;;;; P R O B L E M - A R E A ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(def-class Problem-area (Philosophical-idea)
"++PhiloSURFical : A set of problems linked by different kind of relational schemas, offer organized around a central problem."
((contains-problem :type Problem)
 (has-central-problem :type problem)
 (specified-by-criteria :type thesis)
 (related-to-area :type Problem-area) ;;this should also be inferred, depending on the closure of problems
 (sub-area-of :type Problem-area) 
 (has-sub-area :type Problem-area)))


(def-class Field-of-study (Problem-area)
"++PhiloSURFical : A field of study is seen as a particular kind of problem-area. The distinctive trait of a field of study, is that the area has been socially and historically recognized as separate from the others, and from being a mere agglomerate of problems. Therefore, being fundamentally social constructs, field-of-studies are born and die in history. " 
((defined-by-view :type View)
 (has-exemplar-theory :type theory)
 (has-methodology :type method)))

(def-class generic-field-of-study (Problem-area)
"++PhiloSURFical : Field of study defined extensionally - the definition is not completed yet"
((defined-by-view :type view)))  ;; <---------



#|
(def-rule generic-field-rule
(defined-by-view ?GF ?V) if (generic-field-of-study ?GF)
 (has-sub-area ?GF ?F)
          (defined-by-view ?F ?V)

|#


(def-class Humanistic-discipline (generic-field-of-study)
"++PhiloSURFical : ")

(def-class Scientific-discipline (generic-field-of-study)
"++PhiloSURFical : ")

(def-class Branch-of-philosophy (Humanistic-discipline)
"++PhiloSURFical : The set of established research areas in philosophy, classic subjects of teaching and investigation.We have taken most of them are taken from Wordnet and from http://www.routledge.com/Philosophy/subjects")





;;;;;;;;;;;;; P R O B L E M ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(def-class Problem (Philosophical-idea)
"++PhiloSURFical : the start of a philosophical enquiry"
((contains-concept :type concept)
 (has-problem-type :type problem-type)
 (exists-in-area :type Problem-area)
 (has-supportive-view :type View) 
 (related-to-problem :type Problem)
 (derives-from-problem :type Problem)
 (has-equivalent-meaning-as :type problem)
 (defined-by-argument :type Argument)
 (is-tackled-by-argument :type Argument)
 (is-tackled-by-View :type View) 
 (attacks-View :type View)
 (linked-to-fact :type Temporal-entity)))




(def-class existence-problem (Problem)
"++PhiloSURFical : Problem of the kind 'Does X exist?'"
((contains-concept :type Concept :cardinality 1)))  
(def-class existence-as-concrete-problem (existence-problem)
"++PhiloSURFical : Problem of the kind 'Is X concrete/real?'")
(def-class existence-as-abstract-problem (existence-problem)
"++PhiloSURFical : Problem of the kind 'Is X abstract?'")

(def-class definitory-problem (Problem)
"++PhiloSURFical : Problem of the kind 'What is X?'"
((contains-concept :type Concept :cardinality 1))) 
(def-class definitory-problem-essence (definitory-problem)
"++PhiloSURFical : Problem of the kind 'what are the characteristic traits X has?'")
(def-class definitory-problem-attribute (definitory-problem)
"++PhiloSURFical : Problem of the kind 'what are the attributes X has?'")
(def-class composition-problem (definitory-problem-essence)
"++PhiloSURFical : Problem of the kind 'What is X composed of?")


(def-class functional-problem (Problem)
"++PhiloSURFical : Problem of the kind 'What is the function of X?'"
((contains-concept :type Concept :cardinality 1))) 
(def-class purpose-problem (functional-problem)
"++PhiloSURFical : Problem of the kind 'What is the purpose of X?'") 

(def-class relational-problem (Problem)
"++PhiloSURFical : Problem of the kind 'What is the relation between X and Y?'"
((contains-concept :type Concept :cardinality 2))) 
(def-class dependence-problem (relational-problem)
"++PhiloSURFical : Problem of the kind 'Are X and Y dependent?'")
(def-class dependence-cause-problem (dependence-problem)
"++PhiloSURFical : Problem of the kind 'Is X the cause of Y?'")
(def-class dependence-effect-problem (dependence-problem)
"++PhiloSURFical : Problem of the kind 'Is X the effect of Y?'")
(def-class independence-problem (relational-problem)
"++PhiloSURFical : Problem of the kind 'Is X independent from Y?'")
(def-class equality-problem (relational-problem)
"++PhiloSURFical : Problem of the kind 'Is X equal to Y?'")
(def-class difference-problem (relational-problem)
"++PhiloSURFical : Problem of the kind 'Is X different from Y?'")


(def-class modality-problem (Problem)
"++PhiloSURFical : Problem about the degree of certainty X is likely to happen, or not"
((contains-concept :type Concept :cardinality 1)))
(def-class necessity-problem (modality-problem)
"++PhiloSURFical : Problem of the kind 'is X necessary?'")
(def-class possibility-prolem (modality-problem)
"++PhiloSURFical : Problem of the kind 'is X possible?'")
(def-class contingency-problem (modality-problem)
"++PhiloSURFical : Problem of the kind 'is X contingent?'")
(def-class impossibility-problem (modality-problem)
"++PhiloSURFical : Problem of the kind 'is X impossible?'")

;; inserito per necessita di trovare un problema astratto-generale che sussuma il problema epistemologico
(def-class factual-problem (Problem)
"++PhiloSURFical : Problem of the kind 'how, in what way does X happen, or manifests itself?"
((contains-concept :type Concept :cardinality 1)))






;;;;;;;;;;;;;;;;;;;;;;;;;    A R G U M E N T   S T R U C T U R E   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



(def-class Argument-Structure (Philosophical-idea)
"++PhiloSURFical : Class used to group all the argument-related entities"
((associated-to-view :type View)
 (has-associated-concept :type concept)
 (has-associated-event :type argumentation)
 (has-associated-experiment :type experiment)))



(def-class Argument (Argument-Structure)
"++PhiloSURFical : Reification of the argumentation class - it represents the famous argumentative knots in the history of philosophy"
((uses-method :type argumentative-method)
 (has-argument-part :type argument-part)))


;; need to add all the constraints--> value of the methods used!!! 
(def-class Inductive-argument (Argument)
"++PhiloSURFical : ")
;;((uses-method :type inductive-method)))

(def-class Deductive-argument (Argument)
"++PhiloSURFical : ")
;;((uses-method :type deductive-method)))

(def-class abductive-argument (Argument)
"++PhiloSURFical : ")
;;((uses-method :type abductive-method)))




(def-class Argument-part (Argument-Structure)
"++PhiloSURFical : Statements or sets of statements having a logical value within an argument."
((belongs-to-argument :type Argument)))


(def-class Assumption (Argument-part)
"++PhiloSURFical : Proposition that does not necessarily have a demonstration, but that is taken as true. 
E.g. the assumption of 'goodness of god' in Descartes. In such cases, it is quite close to a view, a principle."
((assumed-in-theory :type Theory)
 (assumed-in-School-of-thought :type School-of-thought)
 (developed-in :type Demonstration)))

(def-class Hypothesis (assumption)
"++PhiloSURFical : Assumption that is used in an argument, specifically in a science-related one (experimental). It implicitly supposes that the hypothesis will be proven true or false by the experiment."
((used-in-experiment :type experiment)))

(def-class Demonstration (Argument-part)
"++PhiloSURFical : The propositions describing the middle steps in the argumentation"
((develops-premise :type Hypothesis)
 (produces-conclusion :type Thesis)))

(def-class Conclusion (Argument-part)
"++PhiloSURFical : Propositions related to the conclusion of an argument. Here there is some implicit overlap with the meaning of thesis, which is a view. We have not formalized this relation yet."
((produced-by-demonstration :type Demonstration)))





;;;;;;;;;;;;;;;;;;;;;;;;;    M E T H O D    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(def-class Method (Philosophical-idea)
"++PhiloSURFical : A recognized philosophical methodology, abstract or practical. It can be mapped to Dolce's procedures"
((is-used-by-view :type View)
 (used-by-argument :type argument)
 (is-defined-by-view :type View))) ;; this is just the inverse of has-Method..


(def-class Abstract-method (Method)
"++PhiloSURFical : A procedure of reasoning or an abstract methodology. For example, the dialectic method in Plato, the epoche' in Husserl, Baco's scientific method or the absolute doubt of Descates")

(def-class Practical-method (Method)
"++PhiloSURFical : A practical doctrine of life, such as the non-violence doctrine of Gandhi, or the ascetism in Schopenauer ")


(def-class logical-mathematical-method (abstract-method)
"++PhiloSURFical : ")

(def-class algorithm (logical-mathematical-method)
"++PhiloSURFical : ")

(def-class rule-of-inference (abstract-method)
"++PhiloSURFical : ")
;; we also have rules-of-replacement!!!! which are still part of the prop calculus.. so maybe all these rule types will be subclasses of log-math-method....

(def-class fallacy (rule-of-inference)
"++PhiloSURFical : ")

(def-class argumentative-method (abstract-method)
"++PhiloSURFical : ")

#| we made these instances
(def-class deductive-method argumentative-method)
(def-class inductive-method argumentative-method)
(def-class abductive-method argumentative-method)
|#


(def-class Precept (Practical-method)  ;; we also called it life-praxis
"++PhiloSURFical : A rule of personal conduct, like a commandment in Christianity or the five precepts in buddism")

(def-class Scientific-method (Practical-method)
"++PhiloSURFical : A rule of personal conduct, like a commandment in Christianity or the five precepts in buddism")




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  R E T H O R I C A L    F I G U R E     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(def-class Rhetorical-figure (Philosophical-idea) 
"++PhiloSURFical : A proposition that uses some literary figure to express or hint at its meaning"
((used-in-argument :type Argument)
 (used-in-View :type View)))

(def-class Maxime (Rhetorical-figure) ;; 
"++PhiloSURFical : Short statement expressing a general truth or rule of conduct. Examples are -amor fati- or -adequatio intellectus ad rem- or -cogito ergo sum-"
((explains-metaphorically :type View)))


(def-class Metaphor (Rhetorical-figure)
"++PhiloSURFical : Any Philosophical-idea that means something in a not-direct manner, e.g. through some analogy"
((explains-metaphorically :type View)))
(def-class Analogy (metaphor)
"++PhiloSURFical : ...")
(def-class Myth (Metaphor)
"++PhiloSURFical : The myth of the cave in Plato"
((explains-metaphorically :type View)))


(def-class Thought-experiment (Rhetorical-figure)
"++PhiloSURFical : The twin earth of Putnam, or the Chinese room of Searle..")

(def-class Example (Rhetorical-figure)
"++PhiloSURFical : ...")
(def-class counter-example (example)
"++PhiloSURFical : ")



















